package DBIx::ConnectInfo;

use Moose;

# Subclass attributes
has 'namespace'     => (is => 'ro', isa => 'Str', required => 1);
has 'dsn_env_map'   => (is => 'ro', isa => 'HashRef', required => 1);
has 'defaults_map'  => (is => 'ro', isa => 'HashRef', required => 1, default => sub { {
  env  => 'dev',
  role => 'web',
} } );

# Standard attributes
has 'env'   => (is => 'ro', isa => 'Str', lazy => 1, default => sub {
  my $self = shift;
  return if $self->{env};
  my $env_var_env = uc($self->{namespace}) . '_ENV';
  $self->{env} = $ENV{ $env_var_env } || $self->{defaults_map}->{env}
    or die "Cannot derive " . __PACKAGE__ . " environment";
});
has 'role'  => (is => 'ro', isa => 'Str', lazy => 1, default => sub {
  my $self = shift;
  return if $self->{role};
  my $env_var_role = uc($self->{namespace}) . '_ROLE';
  $self->{role} = $ENV{ $env_var_role } || $self->{defaults_map}->{role}
    or die "Cannot derive " . __PACKAGE__ . " role";
});
has 'auth'  => (is => 'ro', handles => [qw(user pass)], lazy => 1, default => sub {
  my $self = shift;
  my $auth_class = sprintf '%s::%s::%s',
    ucfirst($self->namespace) . '::Auth', ucfirst $self->role, ucfirst $self->env;
  $self->{auth} = eval "require $auth_class; $auth_class->new"
    or die "Cannot load auth class $auth_class: $@";
});

our $VERSION = '0.02';

=head1 NAME

DBIx::ConnectInfo - Base class for handling DBI connection information 
for one or more database connections.

=head1 SYNOPSIS

    # Used via a subclass:
    package Foobar::ConnectInfo;
    use Moose;
    extends 'DBIx::ConnectInfo';

    # Subclasses should provide defaults for 'namespace' and 'dsn_env_map'
    # and (optionally) for 'defaults_map'
    has '+namespace' => ( default => 'Foobar' );
    has '+dsn_env_map' => ( default => sub { { 
      dev  => 'dbi:Pg:dbname=foobar',
      test => 'dbi:Pg:dbname=foobar_test',
      prod => 'dbi:Pg:dbname=foobar;host=db1',
    } } );
    has '+defaults_map' => ( default => sub { {
      env  => 'dev',
      role => 'web',
    } } );
    1;


    # Usage, elsewhere in your code:
    $ci = Foobar::ConnectInfo->new( role => 'web' );
    $dbh = DBI->connect( $ci->info );
    
    # info() will also instantiate an object if called as a class method e.g.
    $schema = Foobar::Schema->connect( Foobar::ConnectInfo->info( env => 'test' ));


=head1 DESCRIPTION

DBIx::ConnectInfo is a base class for managing connection information
for one or more database connections. It allows you to define one or 
more logical environments ('env') - like dev, test, prod - and map 
those to data sources, and one or more 'roles' (e.g. 'web', 'scripts',
'admin'), and map the combination of environment and role to distinct 
usernames and passwords.

Environment and role are derived at object construction time as follows.

1. If they are provided explicitly as constructor arguments they are
used as given. e.g.

    $ci = Foobar::ConnectInfo->new( env => 'test' );

2. If they are not defined explicitly, DBIx::ConnectInfo checks an
environment variable to see if there are set there. The environment 
variables used are ${namespace}_ENV and ${namespace}_ROLE, in uppercase
e.g. if namespace is 'Foobar', then we check FOOBAR_ENV for 'env', and
FOOBAR_ROLE for 'role'.

3. If either is still not defined, we use the default defined in the
'defaults_map'. The base-class provided default defaults_map is:

    {
      env  => 'dev',
      role => 'web',
    }

but this may be overridden.

=head2 USAGE

To use, you must define a DBIx::ConnectInfo subclass that provides 
defaults for 'namespace' and 'dsn_env_map', and optionally for 
'defaults_map'. 

In addition, you must define one or more subclasses of 
DBIx::Connection::Auth to hold individual username / password pairs.
The naming convention required for these subclasses is:

    NAMESPACE::Auth::ROLE::ENV

e.g. if the namespace is 'Foobar', you might define authentication
subclasses like:

    Foobar::Auth::Web::Dev
    Foobar::Auth::Web::Test
    Foobar::Auth::Web::Prod

It's a bit verbose to have to define a new subclass for each username/
password pair, but doing so allows you to apply different permissions
to the different files for maximum security of the credentials you care
about.

=head1 METHODS

=over 4

=item env

Return the current environment.

=item role

Return the current role.

=item dsn

Return the data source defined for the current environment.

=cut

sub dsn {
  my $self = shift;
  return $self->dsn_env_map->{ $self->env };
}

=item user

Return the username defined for the current environment and role.

=item pass

Return the password defined for the current environment and role.

=item auth

Return the DBIx::ConnectInfo::Auth object defining the username
and password for the current environment and role.

=item info

Return the [ data_source, username, password ] tuple associated with
the current environment and role as an array, suitable for passing 
to a DBI->connect() style method e.g.

    $ci = Foobar::ConnectInfo->new( role => 'web' );
    $dbh = DBI->connect( $ci->info );

If called as a class method, info() will call new() for you to 
instantiate an object with the supplied arguments i.e. the previous
example can be shortened to:

    $dbh = DBI->connect( $ci->info( role => 'web' ) );

=cut

sub info {
  my $self = shift;
  $self = $self->new( @_ ) if ! ref $self;
  return ($self->dsn, $self->user, $self->pass);
}

=item namespace

Return the current namespace, used as a prefix for environment 
variables and auth modules. 

=item dsn_env_map

Return the current data source environment map, a hashref defining
the DBI data source associated with each environment e.g.

    { 
      dev  => 'dbi:Pg:dbname=foobar',
      test => 'dbi:Pg:dbname=foobar_test',
      prod => 'dbi:Pg:dbname=foobar;host=db1',
    }

=item defaults_map

Return the current defaults map, a hashref defining defaults for
'env' and 'role'. Default:

    {
      env  => 'dev',
      role => 'web',
    }

=back

=head1 SEE ALSO

DBIx::ConnectInfo::Auth

=head1 AUTHOR

Gavin Carr, C<< <gavin at openfusion.com.au> >>

=head1 BUGS

Please report any bugs or feature requests to 
C<bug-dbix-connectinfo at rt.cpan.org>, or through
the web interface at 
L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=DBIx-ConnectInfo>.  
I will be notified, and then you'll automatically be notified of 
progress on your bug as I make changes.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc DBIx::ConnectInfo


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=DBIx-ConnectInfo>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/DBIx-ConnectInfo>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/DBIx-ConnectInfo>

=item * Search CPAN

L<http://search.cpan.org/dist/DBIx-ConnectInfo/>

=back


=head1 ACKNOWLEDGEMENTS


=head1 COPYRIGHT & LICENSE

Copyright 2008 Gavin Carr.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1; # End of DBIx::ConnectInfo

__END__

# vim:sw=2:et

